/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

public class Cable.Adapter.Irc.Channel : GLib.Object, Port.Irc.Channel {

    public string server_name { get; private set; }

    public string name { get; set; }

    string _topic = "";
    public string topic {
        get {
            try {
                _topic = client.channel_topic (server_name, this.name);
            } catch (GLib.IOError error) {
                warning (error.message);
            }
            
            return _topic;
        }
    }

    string[] _nicks = {};
    public string[] nicks {
        get {
            try {
                client.channel_nicks (server_name, this.name, out _nicks, out _prefixes);
            } catch (GLib.IOError error) {
                warning (error.message);
            }
            
            return _nicks;
        }
    }

    string[] _prefixes = {};
    public string[] prefixes {
        get {
            try {
                client.channel_nicks (server_name, this.name, out _nicks, out _prefixes);
            } catch (GLib.IOError error) {
                warning (error.message);
            }
            
            return _prefixes;
        }
    }

    public bool joined {
        get {
            try {
                return name in client.channels (server_name);
            } catch (GLib.IOError error) {
                warning (error.message);
            }
            
            return false;
        }
    }

    public Channel (string channel_name, string server_name) {
        this.name = channel_name;
        this.server_name = server_name;
        setup_signals ();
    }

    /**
     * Send a message displayed in "third person"
     * @param message The message to be sent.
     */
    public void send_action (string message) {
        try {
            client.action (server_name, name, message);
        } catch (GLib.IOError error) {
            warning (error.message);
        }
    }

    /**
     * Invite a person to join this channel.
     * @param who Nick name of the person to invite.
     */
    public void send_invite (string who) {
        try {
            client.invite (server_name, name, who);
        } catch (GLib.IOError error) {
            warning (error.message);
        }
    }

    /**
     * Join this channel.
     * @param key Password to this channel. Should be "" if there is no password.
     */
    public void send_join (string key = "") {
        try {
            client.join (server_name, name, key);
        } catch (GLib.IOError error) {
            warning (error.message);
        }
    }

    /**
     * Kick a user from the channel. Requires operator privileges.
     * @param who Nick name of the user to be kicked.
     * @param message Reason why the user has been kicked.
     */
    public void send_kick (string who, string message) {
        try {
            client.kick (server_name, name, who, message);
        } catch (GLib.IOError error) {
            warning (error.message);
        }
    }

    /**
     * Request topic and number of users for this channel TODO move somewhere else
     */
    public void send_list () {
        try {
            client.list (server_name, name);
        } catch (GLib.IOError error) {
            warning (error.message);
        }
    }

    /**
     * Send a normal message in this channel.
     * @param message The message.
     */
    public void send_message (string message) {
        try {
            client.message (server_name, name, message);
        } catch (GLib.IOError error) {
            warning (error.message);
        }
    }

    /**
     * Request a list of names from the channel.
     */
    public void send_names () {
        try {
            client.names (server_name, name);
        } catch (GLib.IOError error) {
            warning (error.message);
        }
    }

    /**
     * Leave this channel.
     * @param message Part message to be displayed. If null it will be fetched from the identity.
     */
    public void send_part (string? message = null) {
        try {
            client.part (server_name, name, message ?? "");
        } catch (GLib.IOError error) {
            warning (error.message);
        }
    }

    /**
     * Change or request the channel's topic.
     * @param topic The new topic, or "" if you just want to query the current topic.
     */
    public void send_topic (string topic = "") {
        try {
            client.topic (server_name, name, topic);
        } catch (GLib.IOError error) {
            warning (error.message);
        }
    }

    /**
     * Get the channel mode for a given user.
     * @param nick Nick name of the user.
     */
    public string user_channel_mode (string nick) {
        try {
            return client.user_channel_mode (server_name, name, nick);
        } catch (GLib.IOError error) {
            warning (error.message);
        }
        
        return "";
    }

    /**
     * Get the channel prefix ("@", "+") of a given user.
     * @param nick Nick name of the user.
     */
    public string user_channel_prefix (string nick) {
        try {
            return client.user_channel_prefix (server_name, name, nick);
        } catch (GLib.IOError error) {
            warning (error.message);
        }
        
        return "";
    }
    
    void setup_signals () {
        client.event_action.connect ((time, server, from, target, message) => {
            if (server == this.server_name && name == target)
                event_action (from, message);
        });
        
        client.event_banlist.connect ((time, server, channel, mask, who, when) => {
            if (server == this.server_name && name == channel)
                event_banlist (mask, who, when);
        });

        client.event_cannot_join.connect ((time, server, channel, reason) => {
            if (server == this.server_name && name == channel)
                event_cannot_join (reason);
        });

        client.event_invite.connect ((time, server, from, channel, who) => {
            if (server == this.server_name && name == channel)
                event_invite (from, who);
        });

        client.event_join.connect ((time, server, from, channel) => {
            if (server == this.server_name && name == channel)
                event_join (from);
        });

        client.event_kick.connect ((time, server, from, channel, who, message) => {
            if (server == this.server_name && name == channel)
                event_kick (from, who, message);
        });

        client.event_list.connect ((time, server, channel, users, topic) => {
            if (server == this.server_name && name == channel)
                event_list (users, topic);
        });
        
        client.event_message.connect ((time, server, from, target, message) => {
            if (server == this.server_name && name == target)
                event_message (from, message);
        });

        client.event_names.connect ((time, server, channel, nicks, prefixes) => {
            if (server == this.server_name && name == channel)
                event_names (nicks, prefixes);
        });

        client.event_no_such.connect ((time, server, target, type) => {
            if (server == this.server_name && name == target && type == "c")
                event_no_such ();
        });

        client.event_part.connect ((time, server, from, channel, message) => {
            if (server == this.server_name && name == channel)
                event_part (from, message);
        });

        client.event_topic.connect ((time, server, from, channel, topic) => {
            if (server == this.server_name && name == channel)
                event_topic (from, topic);
        });
    }
}
