/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

public class Cable.Controller.Window : GLib.Object {

    public Port.View.Window view { get; set; }
    public Port.Irc.Session backend { get; construct; }
    
    public Gee.HashMap <string, Server> servers {
        get; set;
        default = new Gee.HashMap <string, Server> ();
    }

    public Window (Port.View.Window window, Port.Irc.Session session) {
        GLib.Object (view: window, backend: session);
        connect_signals ();
        initialize ();
    }

    void initialize () {
        var channels = Settings.preferences.channels;

        if (channels.length > 0) {
            foreach (var channel in channels) {
                var parts = channel.split (":");
                return_if_fail (parts.length == 3);
                join_channel (parts[0], parts[1], parts[2]);
            }
        }

        if (servers.size == 0)
            view.welcome_shown = true;
    }

    void connect_signals () {
        view.selected.connect (set_window_title_on_chat_switched);
        view.join.connect (join_channel);
        view.remove_channel.connect (remove_chat);
        view.add_channel.connect (show_join_dialog);
        view.show_preferences.connect (show_preferences_dialog);
        view.close.connect (part_all_channels_and_update_settings);
    }
    
    void set_window_title_on_chat_switched (Port.View.Chat chat) {
        view.title = chat.name;
        Settings.preferences.last_used_nick = chat.nick;
        chat.unread_messages = 0;
    }
    
    void part_all_channels_and_update_settings () {
        string[] chans = {};

        foreach (var server in servers.values) {
            foreach (var channel in server.channels.values) {
                chans += string.join (":", server.address, server.nick, channel.name);
                channel.backend.send_part ();
            }
        }

        Settings.preferences.channels = chans;
    }
    
    void show_preferences_dialog () {
        var dialog = new Dialogs.Preferences ();
        dialog.show_all ();
    }
    
    void show_join_dialog () {
        var dialog = new Dialogs.JoinChannel ();
        dialog.done.connect (join_channel);
        dialog.show_all ();
    }
    
    void join_channel (string address, string nick, string channel_name) {
        Server server = null;
        foreach (var s in servers.values)
            if (s.address == address && s.nick == nick)
                server = s;

        if (server == null) {
            server = new Server (address, nick, this);
            servers[server.backend.name] = server;
        }

        if (server.view.has_chat (channel_name)) {
            warning ("Channel is already joined");
            return;
        }

        var channel = new Channel (server, channel_name);
        server.channels[channel_name] = channel;
        
        view.current_chat = channel.view;
        view.welcome_shown = false;
    }
    
    void remove_chat () {
        var channel_id = view.current_chat.id;
        var server_id = view.current_server.id;

        Server server = null;

        if (view.current_chat == null) {
            warning ("No channel selected, cannot leave.");
        } else if (view.current_chat is Port.View.Channel) {
            var channel = servers[server_id].channels[channel_id];
            server = channel.server;
            channel.backend.send_part ();
            server.view.remove_chat (channel.view.id);
            server.channels.unset (channel.view.id);
        } else if (view.current_chat is Port.View.PrivateChat) {
            var channel = servers[server_id].private_chats[channel_id];
            server = channel.server;
            server.view.remove_chat (channel.view.id);
            server.private_chats.unset (channel.view.id);
        }
        
        if (server.channels.size == 0 && server.private_chats.size == 0) {
            servers.unset (server.view.id);
            view.remove_server (server.view.id);
        }

        if (servers.size == 0) {
            view.title = App.TITLE;
            view.welcome_shown = true;
        }
    }
}
